/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.List;

import org.geotools.feature.Feature;

import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraAbschnitt;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraNullpunkt;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Repr&auml;sentiert einen Teilabschnitt eines Strecken-Features der
 * Okstra-Datenbasis (okstra:Strecke.entlang_Teilabschnitt.Teilabschnitt).
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: StreckenTeilAbschnitt.java 10402 2008-07-10 09:07:00Z gieseler $
 * 
 */
public class StreckenTeilAbschnitt extends AsbStationierung {
	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();
	/**
	 * Feature-ID des assoziierten Features vom Typ 'okstra:Strecke'.
	 */
	protected String streckenFeatureID = null;

	/**
	 * Erzeugt einen Streckenteilabschnitt.
	 * 
	 * @param streckenfeatureId
	 *            Feature-Id des assoziierten Features vom Typ 'okstra:Strecke'
	 * @param aoafeature
	 *            Abschnitt oder Ast
	 * @param anfang
	 *            Anfangsstationierung
	 * @param ende
	 *            Endstationierung
	 */
	public StreckenTeilAbschnitt(final String streckenfeatureId,
			final Feature aoafeature, final int anfang, final int ende) {
		streckenFeatureID = streckenfeatureId;
		aoaFeature = aoafeature;
		stationierungAnfang = anfang;
		stationierungEnde = ende;
		bestimmeAsbStationierung();
	}

	/**
	 * Bestimmt die vollst&auml;ndigen Asb-Stationierungsangaben dieses
	 * Streckenteilabschnittes. Dazu werden zun&auml;cht die Nullpunkte auf der
	 * Basis des AoA-Features bestimmt (<code>okstra:Abschnitt-oder-Ast.beginnt_bei_NP,
	 * okstra:Abschnitt-oder-Ast.endet_bei_NP</code>).
	 * Die Nullpunkt-Bezeichnung wird dann aus den ermittelten
	 * Feature-Referenzen ermittelt (<code>okstra:Nullpunkt.Bezeichnung</code>).
	 */
	private void bestimmeAsbStationierung() {
		try {
			String linkNullpunktanfang = (String) aoaFeature
					.getAttribute(OkstraAbschnitt.BEGINNT_BEI_NP);
			Feature nullpunktanfang = Okstra
					.getFeatureReferenz(linkNullpunktanfang);
			anfangsKnoten = (String) nullpunktanfang
					.getAttribute(OkstraNullpunkt.BEZEICHNUNG);

			String linkNullpunktende = (String) aoaFeature
					.getAttribute(OkstraAbschnitt.ENDET_BEI_NP);
			Feature nullpunktende = Okstra
					.getFeatureReferenz(linkNullpunktende);
			endKnoten = (String) nullpunktende
					.getAttribute(OkstraNullpunkt.BEZEICHNUNG);

			offset = 0;
		} catch (ImportException e) {
			DEBUG.error(e.getMessage());
		}
	}

	/**
	 * Bestimmt die Liste der f&uuml;r diesen Teilabschnitt relevanten
	 * Aufteilungspunkte.
	 * 
	 * @param teilungspunkte
	 *            Aufteilungspunkte
	 * @return Liste von Aufteilungspunkten
	 * @throws ImportException
	 *             wenn kein AoA existiert
	 */
	public List<Teilungspunkt> bestimmeTeilungspunkte(
			final TeilungspunkteOkstra teilungspunkte) throws ImportException {
		if (aoaFeature == null) {
			throw new ImportException("kein AoA zu Streckenelement: " + //$NON-NLS-1$
					streckenFeatureID);
		}

		return teilungspunkte.getAoaPunkte(aoaFeature.getID());
	}

	/**
	 * Gibt die ASB-Stationierung dieses Teilabschnittes zur&uuml;ck.
	 * 
	 * @return {@link AsbStationierung}
	 */
	public AsbStationierung getAsbStationierung() {
		return this;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	@SuppressWarnings("nls")
	public String toString() {
		return "Streckenteilabschnitt [AoA: " + aoaFeature.getID()
				+ ", Station Anfang: " + stationierungAnfang
				+ " Station Ende: " + stationierungEnde + "]";
	}
}
