/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig;

import java.util.ArrayList;

import org.geotools.feature.Feature;

import de.bsvrz.kex.bwsin.konfig.okstra.Okstra;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraFeatures;
import de.bsvrz.kex.bwsin.konfig.okstra.OkstraStrecke;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse repr&auml;sentiert ASB-Stationierungsinformationen einer Strecke
 * (okstra:Strecke) als Liste von Streckenteilabschnitten. <br>
 * Eine <code>okstra:Strecke</code> besteht aus einer Liste von
 * Teilabschnitten, von denen jeder eine ASB-Stationierungsinformation
 * darstellt.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: StreckenTeilabschnitte.java 7800 2008-03-28 18:22:26Z gieseler $
 * 
 */
public class StreckenTeilabschnitte extends ArrayList<StreckenTeilAbschnitt> {
	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 2587270235144647475L;

	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Erzeugt die Liste der Teilabschnitte des okstra:Strecke-Features.
	 * 
	 * @param streckenfeatureid
	 *            Feature-ID der Strecke
	 */
	public StreckenTeilabschnitte(final String streckenfeatureid) {

		final Okstra okstradb = Okstra.getInstance();

		final Object[] aoa = okstradb.getSpecialAttributeList2(
				OkstraFeatures.STRECKE, streckenfeatureid, OkstraStrecke.AOA);

		final Object[] sa = okstradb.getSpecialAttributeList2(
				OkstraFeatures.STRECKE, streckenfeatureid,
				OkstraStrecke.STATION_ANFANG);

		final Object[] se = okstradb.getSpecialAttributeList2(
				OkstraFeatures.STRECKE, streckenfeatureid,
				OkstraStrecke.STATION_ENDE);

		if (aoa != null) {
			for (int i = 0; i < aoa.length; i++) {
				Feature aoaFeature = null;
				try {
					aoaFeature = Okstra.getFeatureReferenz(aoa[i]);
					if (aoaFeature == null) {
						debug.error("Die AoA zu " + (String) aoa[i]
								+ " kann nicht bestimmt werden");
						continue;
					}

				} catch (ImportException e) {
					debug.error("Die AoA zu " + (String) aoa[i]
							+ " kann nicht bestimmt werden: " + e.getMessage());
					continue;
				}

				// teste fehlerhafte Stationierung
				final int statanfang = Integer.parseInt((String) sa[i]);
				final int statende = Integer.parseInt((String) se[i]);
				if (statanfang > statende) {
					// throw new ImportException("Die Anfangsstationierung ist
					// grer als die Endstationierung (" + //$NON-NLS-1$
					// streckenfeatureid + ")"); //$NON-NLS-1$
					debug
							.warning("Die Anfangsstationierung ist grer als die Endstationierung (" + //$NON-NLS-1$
									streckenfeatureid + ")"); //$NON-NLS-1$

					// Teilabschnitt mit aufsteigender Stationierung anlegen
					this.add(new StreckenTeilAbschnitt(streckenfeatureid,
							aoaFeature, statende, statanfang));

				} else {
					this.add(new StreckenTeilAbschnitt(streckenfeatureid,
							aoaFeature, statanfang, statende));
				}
			}
		}
	}
}
