/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.aktuell;

import java.util.ArrayList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Polygon;
import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Punkt;

/**
 * Interne Repr&auml;sentation eines Objektes der aktuellen Konfiguration.
 * <p>
 * Die Klasse dient im Wesenlichen der Speicherung des zum Objekt
 * geh&ouml;renden Polygonzuges.
 * </p>
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINAktuellesObjekt.java 10248 2008-07-04 08:19:17Z gieseler $
 * 
 */
public class SINAktuellesObjekt {

	/**
	 * Das zugeh&ouml;rige Systemobjekt.
	 */
	ConfigurationObject davObjekt = null;

	/**
	 * er Polygonzug.
	 */
	WGS84Polygon polygon = null;

	/**
	 * Erzeugt ein neues Objekt auf der Basis eines Systemobjektes.
	 * 
	 * @param objekt
	 *            Systemobjekt
	 */
	public SINAktuellesObjekt(ConfigurationObject objekt) {
		davObjekt = objekt;
	}

	/**
	 * Gibt das zugeh&ouml;rige Systemobjekt zur&uuml;ck.
	 * 
	 * @return Systemobjekt
	 */
	public ConfigurationObject getDavObjekt() {
		return davObjekt;
	}

	/**
	 * Gibt die PID zur&uuml;ck.
	 * 
	 * @return PID
	 */
	public String getPid() {
		return davObjekt.getPid();
	}

	/**
	 * Gibt das Polygon zur&uuml;ck.
	 * 
	 * @return PID
	 */
	public WGS84Polygon getPolygon() {
		if (polygon == null) {
			polygon = new WGS84Polygon(getPunktliste());
		}

		return polygon;
	}

	/**
	 * Erzeugt eine Punktliste aus den Daten der aktuelen Konfiguration.
	 * 
	 * @return Punktliste
	 */
	private ArrayList<WGS84Punkt> getPunktliste() {
		if (davObjekt.isOfType("typ.straenTeilSegment")) {
			return getPunktlisteTeilSegment(davObjekt);
		}

		if (davObjekt.isOfType("typ.straenSegment")) {
			return getPunktlisteSegment();
		}

		return null;
	}

	/**
	 * Liest die Punktkoordinaten eines Stra&szlig;ensegmentes aus und gibt
	 * diese Liste zur&uuml;ck.
	 * 
	 * @return Liste von Punkten
	 */
	public ArrayList<WGS84Punkt> getPunktlisteSegment() {
		ArrayList<WGS84Punkt> punkte = new ArrayList<WGS84Punkt>();

		// doppelte Koordinaten entfernen
		// LinkedHashSet<WGS84Punkt> punkte = new LinkedHashSet<WGS84Punkt>();

		AttributeGroup atg = davObjekt.getDataModel().getAttributeGroup(
				"atg.bestehtAusLinienObjekten"); //$NON-NLS-1$
		Data d = davObjekt.getConfigurationData(atg);

		if (d != null) {
			for (ReferenceValue rv : d
					.getReferenceArray("LinienReferenz").getReferenceValues()) { //$NON-NLS-1$
				ConfigurationObject cro = (ConfigurationObject) davObjekt
						.getDataModel()
						.getObject(rv.getSystemObject().getPid());
				punkte.addAll(getPunktlisteTeilSegment(cro));
			}
		}

		return punkte;
		// return new ArrayList<WGS84Punkt>(punkte);
	}

	/**
	 * Liest die Punktkoordinaten eines Stra&szlig;enteilsegmentes aus und gibt
	 * diese als Liste zur&uuml;ck.
	 * 
	 * @param objekt
	 *            Systemobjekt
	 * @return Liste von Punkten
	 */
	public ArrayList<WGS84Punkt> getPunktlisteTeilSegment(
			ConfigurationObject objekt) {
		ArrayList<WGS84Punkt> punkte = new ArrayList<WGS84Punkt>();

		AttributeGroup ratg = davObjekt.getDataModel().getAttributeGroup(
				"atg.linienKoordinaten"); //$NON-NLS-1$
		Data rd = objekt.getConfigurationData(ratg);
		if (rd != null) {
			Data.NumberArray x = rd.getScaledArray("x"); //$NON-NLS-1$
			Data.NumberArray y = rd.getScaledArray("y"); //$NON-NLS-1$
			int anzahl = x.getLength();

			for (int i = 0; i < anzahl; i++) {
				// WGS84Punkt p = new WGS84Punkt(WGS84Punkt.koordinateRunden(x
				// .getValue(i).doubleValue()), WGS84Punkt
				// .koordinateRunden(y.getValue(i).doubleValue()));
				WGS84Punkt p = new WGS84Punkt(x.getValue(i).doubleValue(), y
						.getValue(i).doubleValue());
				punkte.add(p);
			}
		}

		return punkte;
	}
}
