/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.bwsin.konfig.Location;
import de.bsvrz.kex.bwsin.konfig.lcl.LCL;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;
import de.bsvrz.sys.funclib.kbgen.ProxyDataDescription;

/**
 * SWE-interne Repr&auml;sentation eines inneren Stra&szligensegmentes.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINISS.java 9193 2008-05-26 12:57:36Z gieseler $
 * 
 */
public class SINISS extends AbstraktStrassenSegment {

	/**
	 * Erzeugt einen Namen f&uuml;r ein inneres Stra&szlig;ensegment. Der Name
	 * wird entsprechend Afo gebildet.
	 * 
	 * @param pid
	 *            PID
	 * @param knotenlocation
	 *            {@link Location} des Knotens, zu dem das innere
	 *            Stra&szlig;ensegment geh&ouml;rt.
	 * 
	 * @return Name
	 */
	public static String bildeNamen(String pid, Location knotenlocation) {
		LCL lcl = LCL.getInstance();
		String netzknoten = lcl.getLocationCodeWert(knotenlocation
				.getLocationCode(), LCL.NETZKNOTEN_NR);

		String name = pid + " [NK " + netzknoten + "]"; //$NON-NLS-1$ //$NON-NLS-2$
		return name;
	}

	/**
	 * Das &auml;u&szlig;ere Stra&szlig;ensegment, bei dem das Segment beginnt.
	 */
	private KonfigurationsObjekt vonAss = null;

	/**
	 * Das &auml;u&szlig;ere Stra&szlig;ensegment, bei dem das Segment endet.
	 */
	private KonfigurationsObjekt nachAss = null;

	/**
	 * Die Knotenlocation, zu der das Segment geh&ouml;rt.
	 */
	private Location knotenLocation = null;

	/**
	 * Konstruktor mit PID und Name.
	 * 
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 */
	public SINISS(String pid, String name) {
		super(pid, name);
	}

	/**
	 * Gibt die {@link Location} des Knotens zur&uuml;ck, zu dem dieses
	 * Stra&szlig;ensegment geh&ouml;rt.
	 * 
	 * @return {@link Location}
	 */
	public Location getKnotenLocation() {
		return knotenLocation;
	}

	/**
	 * Gibt das &auml;u&szlig;ere Stra&szlig;ensegment zur&uuml;ck, an dem das
	 * innere Stra&szlig;ensegment endet.
	 * 
	 * @return {@link KonfigurationsObjekt}
	 */
	public KonfigurationsObjekt getNachAss() {
		return nachAss;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getProxyObjekt(de.bsvrz.dav.daf.main.ClientDavConnection)
	 */
	public ProxyConfigurationObject getProxyObjekt(ClientDavConnection dav) {
		ProxyConfigurationObject pco = new ProxyConfigurationObject(pid, name,
				dav.getDataModel().getType("typ.inneresStraenSegment"));

		davtyp = pco.getType();

		// typ.linie
		Data data = createData(dav, "atg.linie", "asp.eigenschaften");
		data.getScaledValue("Lnge").set(getLaenge());
		pco.addConfigurationData(new ProxyDataDescription(data, null));

		// typ.bestehtAusLinienObjekten
		// atg.bestehtAusLinienObjekten
		data = createData(dav, "atg.bestehtAusLinienObjekten",
				"asp.eigenschaften");
		data.setToDefault();
		Data.ReferenceArray lr = data.getReferenceArray("LinienReferenz");
		List<KonfigurationsObjekt> teilsegmente = getTeilSegmente();
		lr.setLength(teilsegmente.size());
		List<String> linienrefs = new ArrayList<String>(teilsegmente.size());
		for (int i = 0; i < teilsegmente.size(); i++) {
			linienrefs.add(teilsegmente.get(i).getPid());
		}
		pco.addConfigurationData(new ProxyDataDescription(data, linienrefs));

		// typ.straenSegment
		// atg.straenSegment
		data = createData(dav, "atg.straenSegment", "asp.eigenschaften");
		data.setToDefault();

		// "gehrtZuStrae"
		List<String> strasserefs = new ArrayList<String>();
		strasserefs.add(zuStrasse == null ? "" : zuStrasse.getPid());

		data.getScaledValue("Lnge").set(getLaenge());
		pco.addConfigurationData(new ProxyDataDescription(data, strasserefs));

		// atg.inneresStraenSegment
		data = createData(dav, "atg.inneresStraenSegment", "asp.eigenschaften");
		List<String> segmentsrefs = new ArrayList<String>();
		segmentsrefs.add(vonAss == null ? "" : vonAss.getPid());
		segmentsrefs.add(nachAss == null ? "" : nachAss.getPid());

		pco.addConfigurationData(new ProxyDataDescription(data, segmentsrefs));

		return pco;
	}

	/**
	 * Gibt das &auml;u&szlig;ere Stra&szlig;ensegment zur&uuml;ck, an dem das
	 * innere Stra&szlig;ensegment beginnt.
	 * 
	 * @return {@link KonfigurationsObjekt}
	 */
	public KonfigurationsObjekt getVonAss() {
		return vonAss;
	}

	/**
	 * Setzt die {@link Location} zu dem dieses Stra&szlig;ensegment
	 * geh&ouml;rt.
	 * 
	 * @param location
	 *            {@link Location}
	 */
	public void setKnotenLocation(Location location) {
		knotenLocation = location;
	}

	/**
	 * Setzt das &auml;u&szlig;ere Stra&szlig;ensegment, an dem das innere
	 * Stra&szlig;ensegment endet.
	 * 
	 * @param ass
	 *            {@link KonfigurationsObjekt}
	 */
	public void setNachAss(KonfigurationsObjekt ass) {
		nachAss = ass;
	}

	/**
	 * Setzt das &auml;u&szlig;ere Stra&szlig;ensegment, an dem das innere
	 * Stra&szlig;ensegment beginnt.
	 * 
	 * @param ass
	 *            {@link KonfigurationsObjekt}
	 */
	public void setVonAss(KonfigurationsObjekt ass) {
		vonAss = ass;
	}

}
