/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.kbgen.ConfigAreaCreator;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;

/**
 * Modul Konfigurationsbereich.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINKonfigurationsBereich.java 10390 2008-07-09 15:06:39Z gieseler $
 * 
 */
public class SINKonfigurationsBereich implements
		SINKonfigurationsBereichInterface {

	/**
	 * Logger f&uuml;r DEBUG-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Liste der Proxy-Objekte.
	 */
	List<ProxyConfigurationObject> proxyObjekte = new LinkedList<ProxyConfigurationObject>();

	/**
	 * Dateiname der Konfigurationsbereichsdatei.
	 */
	private String dateiNameKonfigurationsBereich;

	/**
	 * Konfigurationsverantwortlicher.
	 */
	private String konfigurationsVerantwortlicher;

	/**
	 * PID des Konfigurationsbereiches.
	 */
	private String konfigurationsBereichsPid;

	/**
	 * Name des Konfigurationsbereiches.
	 */
	private String konfigurationsBereichsName;

	/**
	 * Info zum Konfigurationsbereich.
	 */
	private String konfigurationsBereichsInfo;

	/**
	 * Datenverteiler-Verbindung.
	 */
	private ClientDavConnection dav;

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINKonfigurationsBereichInterface#addObjekt(de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt)
	 */
	public void addObjekt(final KonfigurationsObjekt objekt) {
		try {
			proxyObjekte.add(objekt.getProxyObjekt(dav));
		} catch (ImportException e) {
			DEBUG.error(e.getMessage());
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINKonfigurationsBereichInterface#generiereXML()
	 */
	public void generiereXML() {
		ConfigAreaCreator cac = new ConfigAreaCreator(new File(
				dateiNameKonfigurationsBereich), konfigurationsBereichsPid,
				konfigurationsBereichsName, konfigurationsBereichsInfo,
				konfigurationsVerantwortlicher, proxyObjekte);

		DEBUG.info("Der Konfigurationsbereich (XML) wird erzeugt."); //$NON-NLS-1$

		cac.run();

		DEBUG.info("Der Konfigurationsbereich (XML) wurde erzeugt."); //$NON-NLS-1$
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.SINKonfigurationsBereichInterface#initialisiere(de.bsvrz.dav.daf.main.ClientDavConnection,
	 *      java.lang.String, java.lang.String, java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public void initialisiere(final ClientDavConnection davConnection,
			final String dateiNameKonfigurationsBereichp,
			final String konfigurationsVerantwortlicherp,
			final String konfigurationsBereichsPidp,
			final String konfigurationsBereichsNamep,
			final String konfigurationsBereichsInfop) {

		this.dateiNameKonfigurationsBereich = dateiNameKonfigurationsBereichp;
		this.konfigurationsVerantwortlicher = konfigurationsVerantwortlicherp;
		this.konfigurationsBereichsPid = konfigurationsBereichsPidp;
		this.konfigurationsBereichsName = konfigurationsBereichsNamep;
		this.konfigurationsBereichsInfo = konfigurationsBereichsInfop;

		dav = davConnection;
	}

}
