/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.kb;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.bwsin.konfig.lcl.LCL;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.kbgen.ProxyConfigurationObject;
import de.bsvrz.sys.funclib.kbgen.ProxyDataDescription;
import de.bsvrz.sys.funclib.kbgen.ProxySetDescription;

/**
 * SWE-interne Repr&auml;sentation eines Stra&szlig;enknotens.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINStrassenKnoten.java 22535 2010-04-12 11:20:03Z gieseler $
 * 
 */
public final class SINStrassenKnoten extends AbstraktKonfigurationsObjekt {

	/**
	 * Degug-Logger f&uuml;r Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Erzeugt einen Namen f&uuml;r einen Stra&szlig;enknoten. Der Name wird
	 * entsprechend Afo gebildet.
	 * <p> !! Wunsch Herr Keifer, 12.04.2010: Nur Name1 benutzen!
	 * 
	 * @param dav
	 *            {@link ClientDavConnection}
	 * @param location
	 *            Locationcode des Stra&szlig;enknotens
	 * @return Name
	 */
	@SuppressWarnings("nls")
	public static String bildeNamen(ClientDavConnection dav, String location) {
		LCL lcl = LCL.getInstance();

		String typ = knotenBezeichung(dav, location);

		String name1 = lcl.getLocationCodeWert(location, LCL.FIRST_NAME);
//		String name2 = lcl.getLocationCodeWert(location, LCL.SECOND_NAME);
		String netzknoten = lcl
				.getLocationCodeWert(location, LCL.NETZKNOTEN_NR);

//		String name = typ + " " + name1.trim() + " " + name2.trim() + " [NK "
//				+ netzknoten + "]";
		String name = typ + " " + name1.trim() + "  [NK "
			+ netzknoten + "]";

		return name;
	}

	/**
	 * Erzeugt eine PID f&uuml;r einen Stra&szlig;enknoten. Die PID wird
	 * entsprechend Afo gebildet.
	 * 
	 * @param locs
	 *            Liste von Location-Codes
	 * @return PID
	 */
	public static String bildePID(List<String> locs) {
		String pid = "strassenKnoten"; //$NON-NLS-1$
		for (String loc : locs) {
			pid += "." + loc; //$NON-NLS-1$
		}

		return pid;
	}

	/**
	 * Wandelt einen Stra&szlig;enknotentyp aus der LCL in den Typ entsprechend
	 * DatK um.
	 * 
	 * DatK: SonstigerKnoten 0 Sonstiger unspezifizierter Knoten. AutobahnKreuz
	 * 1 Knoten ist ein Autobahnkreuz. AutobahnDreieck 2 Knoten ist ein
	 * Autobahndreieck. AutobahnAnschlussStelle 3 Knoten ist eine
	 * Autobahnanschlussstelle. AutobahnEnde 4 Knoten markiert das Ende der
	 * Autobahn
	 * 
	 * Bei Eintr&auml;gen, die nicht zugeordnet werden k&ouml;nnen, wird der
	 * DatK Wert auf 'SonstigerKnoten' gesetzt.
	 * 
	 * @param locationcode
	 *            Locationcode
	 * @return DAV-Knotentyp
	 */
	private static int davKnotentypVonLcl(String locationcode) {
		LCL lcl = LCL.getInstance();

		int subtyp = Integer.parseInt(lcl.getLocationCodeWert(locationcode,
				LCL.LOCATION_SUBTYP));

		if (subtyp < 1 || subtyp > 4) {
			return 0;
		}

		return subtyp;
	}

	/**
	 * Erzeugt ein neues Stra&szlig;enknoten-Objekt auf der Basis eines
	 * Locationcodes.
	 * 
	 * @param dav
	 *            {@link ClientDavConnection}
	 * @param locationcode
	 *            Locationcode des Stra&szlig;enknotens
	 * @return {@link SINStrasse}
	 */
	public static SINStrassenKnoten erzeugeStrassenKnoten(
			ClientDavConnection dav, String locationcode) {
		List<String> locs = initLocationen(locationcode);

		String pid = bildePID(locs);
		String name = bildeNamen(dav, locationcode);

		SINStrassenKnoten knoten = new SINStrassenKnoten(pid, name);
		knoten.setLocationen(locs);

		return knoten;
	}

	/**
	 * F&uuml;r die Location werden ber die LCL rekursiv alle eventuell
	 * vorhandenen zus&auml;tzlichen Locations fr diesen Knoten ermittelt
	 * (Spalte INTERSECTIONCODE).
	 * 
	 * @param locationcode
	 *            Location des Knotens
	 * @return Liste von Locationen
	 */
	private static List<String> initLocationen(final String locationcode) {
		List<String> locs = new ArrayList<String>();
		locs.add(locationcode);

		LCL lcl = LCL.getInstance();
		String startloc = locationcode;
		do {
			String intersection = lcl.getLocationCodeWert(startloc,
					LCL.INTERSECTIONCODE);
			if (intersection == null || (intersection.length() == 0)
					|| intersection.equals(locationcode)) {
				break;
			}

			locs.add(intersection);
			startloc = intersection;
		} while (!(startloc.length() == 0));

		Collections.sort(locs);

		return locs;
	}

	/**
	 * Bildet die Bezeichnung des Knotens.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * @param location
	 *            Location
	 * @return Knotenbezeichnung
	 */
	private static String knotenBezeichung(final ClientDavConnection dav,
			final String location) {
		LCL lcl = LCL.getInstance();

		String lcltyp = lcl.getLocationCodeWert(location, LCL.LOCATION_TYP);
		String subtyp = lcl.getLocationCodeWert(location, LCL.LOCATION_SUBTYP);

		String tmctyp = lcltyp.substring(0, 1);
		String tmcortstyp = lcltyp.substring(1);

		String tmctypnr = "";

		if (tmctyp.equals("P")) {
			tmctypnr = "1";
		} else if (tmctyp.equals("L")) {
			tmctypnr = "2";
		} else if (tmctyp.equals("G")) {
			tmctypnr = "3";
		} else {
			DEBUG.error("Unbekannte Kennung '" + tmctyp
					+ "' fr Ortstyp in LCL bei Locationcode " + location);
			return "";
		}

		String davtypnr = tmctypnr
				+ String.format("%02d%03d", Integer.parseInt(tmcortstyp),
						Integer.parseInt(subtyp));

		int davstate = Integer.parseInt(davtypnr);

		// die Bezeichung auf dem Attribut 'att.tmcOrtsTyp' des Dav lesen
		DataModel dm = dav.getDataModel();

		IntegerAttributeType att = (IntegerAttributeType) dm
				.getAttributeType("att.tmcOrtsTyp");

		for (IntegerValueState ivs : att.getStates()) {
			if (ivs.getValue() == davstate) {
				String name = ivs.getName();
				return name.substring(name.indexOf(':') + 1);
			}
		}

		return "";
	}

	/**
	 * Liste der zum Knoten geh&ouml;renden Stra&szlig;enelemente.
	 */
	private final ArrayList<KonfigurationsObjekt> segmente = new ArrayList<KonfigurationsObjekt>();

	/**
	 * Liste der Locationen des Knotens.
	 */
	private List<String> locationen = new ArrayList<String>();

	/**
	 * Konstruktor mit PID und Name.
	 * 
	 * @param pid
	 *            PID
	 * @param name
	 *            Name
	 */
	private SINStrassenKnoten(String pid, String name) {
		super(pid, name);
	}

	/**
	 * F&uuml;gt ein (inneres) Stra&szlig;ensegment hinzu.
	 * 
	 * @param segment
	 *            {@link KonfigurationsObjekt}
	 */
	public void addSegment(KonfigurationsObjekt segment) {
		segmente.add(segment);
	}

	/**
	 * Gibt die Liste der Locationen des Stra&szlig;enknotens zur&uuml;ck.
	 * 
	 * @return Liste der Knotenlocationen
	 */
	public List<String> getLocationen() {
		return locationen;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.kex.bwsin.konfig.kb.KonfigurationsObjekt#getProxyObjekt(de.bsvrz.dav.daf.main.ClientDavConnection)
	 */
	public ProxyConfigurationObject getProxyObjekt(ClientDavConnection dav) {
		ProxyConfigurationObject pco = new ProxyConfigurationObject(pid, name,
				dav.getDataModel().getType("typ.straenKnoten"));

		davtyp = pco.getType();

		// atg.strae
		Data data = createData(dav, "atg.straenKnoten", "asp.eigenschaften");
		data.setToDefault();
		data.getUnscaledValue("Typ").set(davKnotentypVonLcl(locationen.get(0)));
		pco.addConfigurationData(new ProxyDataDescription(data, null));

		// Menge InnereStraenSegmente
		List<String> iss = new ArrayList<String>();
		for (KonfigurationsObjekt ko : segmente) {
			iss.add(ko.getPid());
		}

		pco.addSet(new ProxySetDescription("InnereStraenSegmente", iss));

		return pco;
	}

	/**
	 * Setzt die Locationen des Knotens.
	 * 
	 * @param locationen
	 *            Locationen
	 */
	public void setLocationen(List<String> locationen) {
		this.locationen = locationen;
	}
}
