/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.okstra;

import de.bsvrz.kex.bwsin.konfig.ImportException;
import de.bsvrz.kex.bwsin.konfig.Verkehrsrichtung;

/**
 * Klasse mit statischen Konstanten zum Zugriff auf die Attribute des
 * Feature-Typs 'vrz3:UeberholverbotVRZ'.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: OkstraUeberholverbot.java 10086 2008-07-01 18:36:02Z gieseler $
 * 
 */
@SuppressWarnings("nls")
public class OkstraUeberholverbot extends OkstraFeature {

	/**
	 * Konstante zum Zugriff auf das Attribut "hat_Strecke".
	 */
	public static final String STRECKE = "hat_Strecke";

	/**
	 * Konstante zum Zugriff auf das Attribut
	 * "okstra:Fahrzeugart.okstra:Kennung".
	 */
	public static final String FAHRZEUGART = "okstra:Fahrzeugart.okstra:Kennung";

	/**
	 * Konstante zum Zugriff auf das Attribut "okstra:im_Zeitraum".
	 */
	public static final String ZEITRAUM = "im_Zeitraum";

	/**
	 * Bestimmt die Verkehrsrichtung aus der OKSTRA-Richtungsangabe.
	 * 
	 * @param fid
	 *            Feature-ID
	 * @return Verkehrsrichtung
	 * @throws ImportException
	 *             wenn die Richtung nicht bestimmt werden kann
	 */
	public static Verkehrsrichtung bestimmeVerkehrsRichtung(final String fid)
			throws ImportException {
		return OkstraVerkehrsRichtung.bestimmeVerkehrsRichtung(
				OkstraFeatures.UEBERHOLVERBOT, fid);
	}

	/**
	 * Konstruktor.
	 */
	public OkstraUeberholverbot() {
		super("vrz3:UeberholverbotVRZ", "berholverbot", "vrz3:", Boolean.TRUE);
	}
}
