/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.konfig.okstra.parser;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.naming.OperationNotSupportedException;

import org.geotools.xml.XMLElementHandler;
import org.geotools.xml.schema.Element;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Feature-Handler WFS-Parser.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINFeatureHandler.java 10428 2008-07-10 12:09:46Z gieseler $
 * 
 */
public class SINFeatureHandler extends XMLElementHandler {

	// /**
	// */
	// private static class DefaultElementValue implements ElementValue {
	// private String value = "";
	// private final Element t;
	//
	// /**
	// * Stores the two values for use within the specified type
	// *
	// * @param value
	// * String
	// * @param t
	// * Element
	// *
	// * @see SimpleElementHandler#endElement(String, String)
	// */
	// public DefaultElementValue(String value, Element t) {
	// this.value = value;
	// this.t = t;
	// }
	//
	// /**
	// * @see org.geotools.xml.xsi.ElementValue#getElement()
	// */
	// public Element getElement() {
	// return t;
	// }
	//
	// /**
	// * @see org.geotools.xml.xsi.ElementValue#getValue()
	// */
	// public Object getValue() {
	// return value;
	// }
	// }

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Element.
	 */
	private Element elem;

	/**
	 * Text.
	 */
	private String text = "";

	/**
	 * Der Wert.
	 */
	private final Object value = null;

	/**
	 * Namespace-URI.
	 */
	private String namespaceURI;

	/**
	 * Name.
	 */
	private String localName;

	/**
	 * Attribute des Features.
	 */
	private Attributes attrs;

	/**
	 * Liste der Elementhandler.
	 */
	List<SINElementHandler> elist = new ArrayList<SINElementHandler>();

	/**
	 * Konstruktor.
	 */
	public SINFeatureHandler() {
		/**
		 * 
		 */
	}

	/**
	 * F&uuml;gt einen Elementhandler hinzu.
	 * 
	 * @param seh
	 *            Elementhandler
	 */
	public void add(final SINElementHandler seh) {
		elist.add(seh);
	}

	/**
	 * Erzeugt das Feature.
	 */
	public void buildFeature() {
		// System.out.println( "Root-Element: " + elist.get(0).getName());
		AttributesImpl atts = elist.get(0).getAttribute();
		for (int i = 0; i < atts.getLength(); i++) {
			// System.out.println("Localname: "+atts.getLocalName(i));
			// System.out.println("Qname: "+atts.getQName(i));
			// System.out.println("Type: "+atts.getType(i));
			// System.out.println("Value: "+atts.getValue(i));
			if (atts.getQName(i).equals("fid")) {
				localName = atts.getValue(i);
			}
		}
		// System.out.println("Feature: " + _localName);

		// for(SINElementHandler seh : elist){
		// System.out.println( "Element: " + seh.getName());
		// }
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.xml.XMLElementHandler#characters(java.lang.String)
	 */
	@Override
	public void characters(final String text1) {
		if (this.text != null) {
			this.text = this.text.concat(text1);
		} else {
			this.text = text1;
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.xml.XMLElementHandler#endElement(java.net.URI,
	 *      java.lang.String, java.util.Map)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void endElement(URI namespaceuri, String localname, Map hints)
			throws OperationNotSupportedException, SAXException {
		text = (text == null) ? null : text.trim();

		// ElementValue[] vals;
		// vals = new ElementValue[1];
		// vals[0] = new DefaultElementValue(text, elem);
		// value = type.getValue(elem, vals, attr, hints);
		// attr = null;
		// text = null;
	}

	/**
	 * Gibt den Attributwert zu einem Attributnamen zur&uuml;ck.
	 * 
	 * @param attname
	 *            Attributname
	 * @return Attributwert
	 */
	public String getAttributeWert(String attname) {
		for (SINElementHandler eh : elist) {
			if (eh.getName().equals(attname)) {
				return eh.getText();
			}
		}

		return "";
	}

	/**
	 * Gibt eine Liste von Attributwerten zu einem Attributnamen zur&uuml;ck.
	 * 
	 * @param attname
	 *            Attributname
	 * @return Liste von Attributwerten
	 */
	public List<Object> getAttributeWerte(final String attname) {
		int index = attname.indexOf(":");
		String localname;
		localname = null;
		if (index >= 0) {
			localname = attname.substring(index + 1);
			attname.substring(0, index);
		} else {
			localname = attname;
		}
		List<Object> werte = new ArrayList<Object>();
		for (SINElementHandler eh : elist) {
			if (eh.getName().equals(attname) || eh.getName().equals(localname)) {
				werte.add(eh.getValue());
			}
		}

		return werte;
	}

	/**
	 * Gibt die Attribute des Features zur&uuml;ck.
	 * 
	 * @return Attribute
	 */
	protected Attributes getAttrs() {
		return attrs;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.xml.XMLElementHandler#getElement()
	 */
	@Override
	public Element getElement() {
		return elem;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.xml.XMLElementHandler#getHandler(java.net.URI,
	 *      java.lang.String, java.util.Map)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public XMLElementHandler getHandler(URI namespaceuri, String localname,
			Map hints) throws SAXException {
		throw new SAXException(
				"Should not have any children - this is a simpleType");
	}

	/**
	 * Gibt das letzte Element zur&uuml;ck.
	 * 
	 * @return Elementhandler
	 */
	public SINElementHandler getLast() {
		return elist.get(elist.size() - 1);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.xml.XMLElementHandler#getName()
	 */
	@Override
	public String getName() {
		return localName;
	}

	/**
	 * Gibt die Namespace-URI des Elementes zur&uuml;ck.
	 * 
	 * @return Namespace-URI
	 */
	protected String getNamespaceURI() {
		return namespaceURI;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.xml.XMLElementHandler#getValue()
	 */
	@Override
	public Object getValue() {
		return value;
	}

	/**
	 * startElement-Callback.
	 * 
	 * @param uri
	 *            URI
	 * @param localname
	 *            Name
	 * @param atts
	 *            Attribute
	 */
	public void startElement(final String uri, final String localname,
			final Attributes atts) {
		// this.attr = new AttributesImpl(attr1);
		namespaceURI = uri;
		localName = localname;
		attrs = atts;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.xml.XMLElementHandler#startElement(java.net.URI,
	 *      java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(URI namespaceuri, String localname, Attributes atts)
			throws SAXException {
		/**
		 * 
		 */
	}
}
