/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.vew;

import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.WindowConstants;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Graphische Ausgabe der Log-Meldungen.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 * 
 */
public class LogFenster extends JFrame {

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * F&uuml;gt eine Komponente hinzu.
	 * 
	 * @param cont
	 *            Container
	 * @param gbl
	 *            Gridbaglayout
	 * @param c
	 *            Komponente
	 * @param x
	 *            X-Koordinate
	 * @param y
	 *            Y-Koordinate
	 * @param width
	 *            Breie
	 * @param height
	 *            H&ouml;he
	 * @param weightx
	 *            horizontale Verteilung
	 * @param weighty
	 *            vertikale Verteilung
	 * @param fill
	 *            F&uuml;llung
	 */
	private static void addComponent(Container cont, GridBagLayout gbl,
			Component c, int x, int y, int width, int height, double weightx,
			double weighty, int fill) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(10, 10, 10, 10);
		gbc.fill = fill;
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = width;
		gbc.gridheight = height;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		gbl.setConstraints(c, gbc);
		cont.add(c);
	}

	/**
	 * Font-Style.
	 */
	int fontStyle = Font.PLAIN;

	/**
	 * Font-Size.
	 */
	int fontSize = 12;

	/**
	 * Breite des Fensters.
	 */
	int breite = 700;

	/**
	 * Houml;he des Fensters.
	 */
	int hoehe = 500;
	/**
	 * Layout.
	 */
	private final GridBagLayout gridbaglayout;

	/**
	 * Ausgabe-Pane.
	 */
	private final JScrollPane ausgabePane;

	/**
	 * Schliessen-Button.
	 */
	private final JButton schliessenButton;

	/**
	 * Button-Panel.
	 */
	private final JPanel buttonPanel;

	/**
	 * Der Frame.
	 */
	private final JFrame frame;

	/**
	 * Parent-Frame.
	 */
	private final BWSINgui parent;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            Parent-Frame
	 */
	public LogFenster(final BWSINgui parent) {
		super("Der Import wird ausgefhrt");
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		frame = this;
		this.parent = parent;

		final JTextArea ta = new JTextArea("", 80, 25);
		ta.setEditable(false);
		ta.setLineWrap(true);
		ta.setFont(new Font("Monospaced", fontStyle, fontSize));

		setFont(new Font(null, fontStyle, fontSize));
		ausgabePane = new JScrollPane(ta);
		// this.frame.setUndecorated(true);

		schliessenButton = new JButton("Schliessen"); //$NON-NLS-1$
		schliessenButton.setEnabled(false);

		buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		buttonPanel.add(this.schliessenButton);

		this.gridbaglayout = new GridBagLayout();
		setLayout(this.gridbaglayout);

		addComponent(this, this.gridbaglayout, ausgabePane, 0, 0,
				GridBagConstraints.RELATIVE, 1, 0.5, 2.5,
				GridBagConstraints.BOTH);
		addComponent(this, this.gridbaglayout, this.buttonPanel, 0, 1,
				GridBagConstraints.RELATIVE, 1, 0.5, 0.0,
				GridBagConstraints.BOTH);

		pack();
		setSize(new Dimension(breite, hoehe));
		setLocation(this.getLocation().x
				+ ((this.getSize().width - breite) / 2), this.getLocation().y
				+ ((this.getSize().height - hoehe) / 2));
		setAlwaysOnTop(true);
		// in die Mitte
		int sw = Toolkit.getDefaultToolkit().getScreenSize().width;
		int sh = Toolkit.getDefaultToolkit().getScreenSize().height;
		setLocation((sw - getSize().width) / 2, (sh - getSize().height) / 2);

		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		setVisible(true);

		PrintStream p = new PrintStream(new OutputStream() {
			@Override
			public void write(int b) {
				int convb = b;
				// die Kodierung ist der Umlaute ist unklar, da tun wir es
				// eben selbst
				switch (convb) {
				case -28:
					convb = '';
					break;
				case -60:
					convb = '';
					break;
				case -10:
					convb = '';
					break;
				case -42:
					convb = '';
					break;
				case -4:
					convb = '';
					break;
				case -36:
					convb = '';
					break;
				case -33:
					convb = '';
					break;

				default:
					break;
				}
				ta.append(String.valueOf((char) convb));
				if ((char) convb == '\n') {
					ta.repaint();
					ta.setCaretPosition(ta.getText().length());
				}
			}
		});
		System.setErr(p);
		System.setOut(p);

		// Logger logger;
		// // LogRecord logRecord;
		// // boolean result;
		// Class<?> klasse = BWSIN.class;
		//
		// logger = Logger.getLogger(klasse.getSimpleName() + "."
		// + klasse.getPackage().getName() + "." + klasse.getSimpleName());
		// // logRecord = new LogRecord(level, "");
		// // result = false;
		// logger.addHandler(new ConsoleHandler());
		//
		// for (Handler h :
		// Logger.getLogger(klasse.getSimpleName()).getHandlers()) {
		// if (h instanceof ConsoleHandler) {
		// // result = logger.isLoggable(level) && h.isLoggable(logRecord);
		// break;
		// }
		// }
		// for (Enumeration e = LogManager.getLogManager().getLoggerNames(); e
		// .hasMoreElements();) {
		// String loggerName = e.nextElement().toString();
		// Level level = Logger.getLogger(loggerName).getLevel();
		// }

		Debug.reinstallConsoleHandler();

		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				if (schliessenButton.isEnabled()) {
					schliessen();
				}
			}
		});
	}

	/**
	 * Gibt eine Komponente frei oder sperrt sie.
	 * 
	 * @param cont
	 *            Container
	 * @param enable
	 *            Freigabe ja/nein
	 */
	void enableComponent(final Container cont, final boolean enable) {
		for (Component c : cont.getComponents()) {
			c.setEnabled(enable);
			if (c instanceof Container) {
				enableComponent((Container) c, enable);
			}
		}
	}

	/**
	 * Enabled den Schliessen-Button.
	 */
	public void enableSchliessen() {
		schliessenButton.setEnabled(true);
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

		schliessenButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				schliessen();
			}
		});
	}

	/**
	 * Schlie&szlig;t das Fenster.
	 */
	private void schliessen() {
		parent.stopWorker();
		frame.setVisible(false);
	}
}
