/*
 * Segment 2 (KEx),  SWE 2.Statischer Import Netz
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.bwsin.vew;

import java.util.Properties;

import de.bsvrz.sys.funclib.bitctrl.geolib.GeoTransformation;

/**
 * Klasse mit allen f&uuml;r die Applikation notwendigen Parametern.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: SINProperties.java 10429 2008-07-10 12:09:58Z gieseler $
 * 
 */
public final class SINProperties extends Properties {
	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 1L;

	/** Einziges Objekt dieser Klasse (Singleton). */
	private static SINProperties defaultProperties = new SINProperties();

	/**
	 * Aufrufparameter 'mit GUI'.
	 */
	public static final String PROP_KEY_WITH_GUI = "-guiStatischerImportNetz";

	/**
	 * Aufrufparameter 'Parameter Okstra'.
	 */
	public static final String PROP_KEY_QUELLDATENBASIS = "-netzDatenQuelleParameter";

	/**
	 * Aufrufparameter 'LCL'.
	 */
	public static final String PROP_KEY_LCL = "-dateiNameLcl";

	/**
	 * Aufrufparameter 'Pfad Konfigurationsbereich'.
	 */
	public static final String PROP_KEY_KBPFAD = "-pfadNameKonfigurationsBereich";

	/**
	 * Aufrufparameter 'Konfigurationsverantwortlicher'.
	 */
	public static final String PROP_KEY_KV = "-KonfigurationsVerantwortlicher";

	/**
	 * Aufrufparameter 'PID Konfigurationsbereich'.
	 */
	public static final String PROP_KEY_KBPID = "-KonfigurationsBereichsPid";

	/**
	 * Aufrufparameter 'Name Konfigurationsbereich'.
	 */
	public static final String PROP_KEY_KBNAME = "-KonfigurationsBereichsName";

	/**
	 * Aufrufparameter 'Info Konfigurationsbereich'.
	 */
	public static final String PROP_KEY_KBINFO = "-KonfigurationsBereichsInfo";

	/**
	 * Aufrufparameter 'mit Abweichungen'.
	 */
	public static final String PROP_KEY_ABW = "-mitAbweichungen";

	/**
	 * Aufrufparameter 'Protokoll Abweichungen'.
	 */
	public static final String PROP_KEY_ABW_PROTOKOLLDATEI = "-dateiNameProtokoll";

	/**
	 * Aufrufparameter 'zu testende Netzreferenzen'.
	 */
	public static final String PROP_KEY_REFERENZEN = "-netzReferenzen";

	/**
	 * Aufrufparameter 'genriere TMS-Ortsfreferenzen'.
	 */
	public static final String PROP_KEY_TMCREFERENZEN = "-generiereTMCOrtsReferenzen";

	/**
	 * Aufrufparameter 'max. zul&auml;ssige Abweichung'.
	 */
	public static final String PROP_KEY_MAX_ABWEICHUNG = "-maxZulAbweichung";

	/**
	 * zus&auml;tzliche Properties
	 */
	/**
	 * mit erweiterten Angaben im Objektnamen.
	 */
	public static final String PROP_KEY_XMLDEBUG = "XMLmitDebugInfos";

	/**
	 * Bundesland-Kennung f&uuml;r die Stra&szlg;enteilsegmente.
	 */
	public static final String PROP_KEY_BUNDESLAND = "bundesland";

	/**
	 * Standardwert OKSTRA-Parameterdatei.
	 */
	public static final String DEFAULT_OKSTRA_PARAMETERFILE = "de/bsvrz/kex/bwsin/wfs_bw_Default.properties";

	/**
	 * Standardwert zu pr&uuml;fende Netzreferenzen.
	 */
	public static final String DEFAULT_NETZREFERENZEN = "de.bsvrz.kex.bwsin.NetzReferenzenDefault";

	/**
	 * Standardwert max. zul Abweichung.
	 */
	public static final String DEFAULT_MAX_ABWEICHUNG = "0.000010";

	/**
	 * Bestimmt das einziges Objekt dieser Klasse (Singleton).
	 * 
	 * @return Einziges Objekt dieser Klasse (Singleton).
	 */
	public static SINProperties getInstance() {
		return defaultProperties;
	}

	/**
	 * Privater Konstruktor.
	 */
	private SINProperties() {
		setProperty(PROP_KEY_XMLDEBUG, "nein"); // keine Debuginformationen in
		// der XML-Datei
		setProperty(PROP_KEY_BUNDESLAND, "BW"); // Standardwert Bundesland (fr
		// STS-PID und Name)
		setProperty(PROP_KEY_REFERENZEN, DEFAULT_NETZREFERENZEN);
		setProperty(PROP_KEY_QUELLDATENBASIS, DEFAULT_OKSTRA_PARAMETERFILE);
		setProperty(PROP_KEY_ABW, "nein");
		setProperty(PROP_KEY_MAX_ABWEICHUNG, DEFAULT_MAX_ABWEICHUNG);
		setProperty(PROP_KEY_TMCREFERENZEN, "ja");
		setProperty(PROP_KEY_WITH_GUI, "nein");
	}

	/**
	 * Gibt die maximal zul&auml;ssige Abweichung in Grad zur&uuml;;ck.
	 * 
	 * @return max. Abweichung in Grad
	 */
	public double getMaximaleAbweichungGrad() {
		return Double
				.parseDouble(getProperty(SINProperties.PROP_KEY_MAX_ABWEICHUNG));
	}

	/**
	 * Gibt die maximal zul&auml;ssige Abweichung in Meter zur&uuml;;ck.
	 * 
	 * @return max. Abweichung in Meter
	 */
	public double getMaximaleAbweichungMeter() {
		return GeoTransformation
				.winkelInMeter(Double
						.parseDouble(getProperty(SINProperties.PROP_KEY_MAX_ABWEICHUNG)));
	}

	/**
	 * Hilfsmethode zum Bestimmen, ob ein bin&auml;rer Parameter (ja/nein) auf
	 * 'ja' gesetzt ist.
	 * 
	 * @param key
	 *            Property-Schl&uuml;ssel.
	 * 
	 * @return true, wenn der Wert der Property zum Schl&uuml;ssel 'Ja' ist,
	 *         ansonsten false.
	 */
	public boolean istAktiv(String key) {
		return defaultProperties.getProperty(key).equalsIgnoreCase("Ja");
	}
}
